#ifndef __WORLD_H
#define __WORLD_H

/*
	WORLD.H

  	The CWorld class

	Author: Kevin Hawkins
	Date: 3/29/2001
	Description: The CWorld class represents the game world
			   of the engine. All objects, the terrain,
			   audio system, ambient world music and sound,
			   and the camera are held here.


*/

#include "object.h"
#include "camera.h"
#include "tree.h"
//#include "sphere.h"
#include "sun.h"
#include "earth.h"
#include "emoon.h"
#include "mars.h"
#include "mmoon.h"

class CWorld
{
protected:
     void OnAnimate(float deltaTime);
     void OnDraw(CCamera *camera);
     void OnPrepare();

public:
     HWND hwnd;

	 Sun *sun;
	 Earth *earth;
	 EMoon *emoon;
	 Mars *mars;
	 MMoon *mmoon;
     //CSphere *marsmoon;
     CCamera *camera;              // the camera

   //  CWorld();
     CWorld(CCamera *c);
     ~CWorld();

     // initialize terrain, load objects and put in container
     void LoadWorld();
     void UnloadWorld();

     // do physics calculations for all objects in the world
     // including collisions
     void Animate(float deltaTime);

     // render all objects in the world
     void Draw(CCamera *camera);

     void Prepare() { OnPrepare(); }
};


#endif
